# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1146, 499)
        self.centralWidget = QtWidgets.QWidget(MainWindow)
        self.centralWidget.setObjectName("centralWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralWidget)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.scrollArea = QtWidgets.QScrollArea(self.centralWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scrollArea.sizePolicy().hasHeightForWidth())
        self.scrollArea.setSizePolicy(sizePolicy)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 1126, 327))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.scrollAreaWidgetContents)
        self.horizontalLayout_5.setContentsMargins(11, 11, 11, 11)
        self.horizontalLayout_5.setSpacing(6)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.labelSelectPoint = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelSelectPoint.sizePolicy().hasHeightForWidth())
        self.labelSelectPoint.setSizePolicy(sizePolicy)
        self.labelSelectPoint.setText("")
        self.labelSelectPoint.setObjectName("labelSelectPoint")
        self.horizontalLayout_5.addWidget(self.labelSelectPoint)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout.addWidget(self.scrollArea)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(6)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.btnAutoDetect = QtWidgets.QPushButton(self.centralWidget)
        self.btnAutoDetect.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnAutoDetect.sizePolicy().hasHeightForWidth())
        self.btnAutoDetect.setSizePolicy(sizePolicy)
        self.btnAutoDetect.setObjectName("btnAutoDetect")
        self.horizontalLayout_2.addWidget(self.btnAutoDetect)
        self.label = QtWidgets.QLabel(self.centralWidget)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.spbPagesPerSheet = QtWidgets.QSpinBox(self.centralWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spbPagesPerSheet.sizePolicy().hasHeightForWidth())
        self.spbPagesPerSheet.setSizePolicy(sizePolicy)
        self.spbPagesPerSheet.setMinimum(1)
        self.spbPagesPerSheet.setProperty("value", 6)
        self.spbPagesPerSheet.setObjectName("spbPagesPerSheet")
        self.horizontalLayout_2.addWidget(self.spbPagesPerSheet)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.btnReload = QtWidgets.QPushButton(self.centralWidget)
        self.btnReload.setEnabled(False)
        self.btnReload.setObjectName("btnReload")
        self.horizontalLayout_2.addWidget(self.btnReload)
        self.labelPageNum = QtWidgets.QLabel(self.centralWidget)
        self.labelPageNum.setText("")
        self.labelPageNum.setObjectName("labelPageNum")
        self.horizontalLayout_2.addWidget(self.labelPageNum)
        self.btnPrevious = QtWidgets.QPushButton(self.centralWidget)
        self.btnPrevious.setEnabled(False)
        self.btnPrevious.setObjectName("btnPrevious")
        self.horizontalLayout_2.addWidget(self.btnPrevious)
        self.btnNext = QtWidgets.QPushButton(self.centralWidget)
        self.btnNext.setEnabled(False)
        self.btnNext.setObjectName("btnNext")
        self.horizontalLayout_2.addWidget(self.btnNext)
        self.btnZoomIn = QtWidgets.QPushButton(self.centralWidget)
        self.btnZoomIn.setObjectName("btnZoomIn")
        self.horizontalLayout_2.addWidget(self.btnZoomIn)
        self.btnZoomOut = QtWidgets.QPushButton(self.centralWidget)
        self.btnZoomOut.setObjectName("btnZoomOut")
        self.horizontalLayout_2.addWidget(self.btnZoomOut)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setSpacing(6)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.labelWidthHeight = QtWidgets.QLabel(self.centralWidget)
        self.labelWidthHeight.setObjectName("labelWidthHeight")
        self.horizontalLayout_3.addWidget(self.labelWidthHeight)
        self.spbWidth = QtWidgets.QSpinBox(self.centralWidget)
        self.spbWidth.setMaximum(9999)
        self.spbWidth.setObjectName("spbWidth")
        self.horizontalLayout_3.addWidget(self.spbWidth)
        self.spbHeight = QtWidgets.QSpinBox(self.centralWidget)
        self.spbHeight.setMaximum(9999)
        self.spbHeight.setObjectName("spbHeight")
        self.horizontalLayout_3.addWidget(self.spbHeight)
        self.btnWidthHeight = QtWidgets.QPushButton(self.centralWidget)
        self.btnWidthHeight.setObjectName("btnWidthHeight")
        self.horizontalLayout_3.addWidget(self.btnWidthHeight)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.label_2 = QtWidgets.QLabel(self.centralWidget)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.comboPosition = QtWidgets.QComboBox(self.centralWidget)
        self.comboPosition.setObjectName("comboPosition")
        self.horizontalLayout_3.addWidget(self.comboPosition)
        self.label_3 = QtWidgets.QLabel(self.centralWidget)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_3.addWidget(self.label_3)
        self.spbPositionX = QtWidgets.QSpinBox(self.centralWidget)
        self.spbPositionX.setMaximum(9999)
        self.spbPositionX.setObjectName("spbPositionX")
        self.horizontalLayout_3.addWidget(self.spbPositionX)
        self.label_4 = QtWidgets.QLabel(self.centralWidget)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_3.addWidget(self.label_4)
        self.spbPositionY = QtWidgets.QSpinBox(self.centralWidget)
        self.spbPositionY.setMaximum(9999)
        self.spbPositionY.setObjectName("spbPositionY")
        self.horizontalLayout_3.addWidget(self.spbPositionY)
        self.btnPosition = QtWidgets.QPushButton(self.centralWidget)
        self.btnPosition.setObjectName("btnPosition")
        self.horizontalLayout_3.addWidget(self.btnPosition)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        MainWindow.setCentralWidget(self.centralWidget)
        self.menuBar = QtWidgets.QMenuBar(MainWindow)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 1146, 32))
        self.menuBar.setObjectName("menuBar")
        self.menu_File = QtWidgets.QMenu(self.menuBar)
        self.menu_File.setObjectName("menu_File")
        self.menu_Help = QtWidgets.QMenu(self.menuBar)
        self.menu_Help.setObjectName("menu_Help")
        MainWindow.setMenuBar(self.menuBar)
        self.mainToolBar = QtWidgets.QToolBar(MainWindow)
        self.mainToolBar.setObjectName("mainToolBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.mainToolBar)
        self.statusBar = QtWidgets.QStatusBar(MainWindow)
        self.statusBar.setObjectName("statusBar")
        MainWindow.setStatusBar(self.statusBar)
        self.action_Open = QtWidgets.QAction(MainWindow)
        self.action_Open.setObjectName("action_Open")
        self.action_Save = QtWidgets.QAction(MainWindow)
        self.action_Save.setObjectName("action_Save")
        self.actionAbout_Qt = QtWidgets.QAction(MainWindow)
        self.actionAbout_Qt.setObjectName("actionAbout_Qt")
        self.action_About = QtWidgets.QAction(MainWindow)
        self.action_About.setObjectName("action_About")
        self.action_Quit = QtWidgets.QAction(MainWindow)
        self.action_Quit.setObjectName("action_Quit")
        self.action_Website = QtWidgets.QAction(MainWindow)
        self.action_Website.setObjectName("action_Website")
        self.action_Donate = QtWidgets.QAction(MainWindow)
        self.action_Donate.setObjectName("action_Donate")
        self.action_Project_page = QtWidgets.QAction(MainWindow)
        self.action_Project_page.setObjectName("action_Project_page")
        self.action_Blog = QtWidgets.QAction(MainWindow)
        self.action_Blog.setObjectName("action_Blog")
        self.menu_File.addAction(self.action_Open)
        self.menu_File.addAction(self.action_Save)
        self.menu_File.addAction(self.action_Quit)
        self.menu_Help.addAction(self.action_Website)
        self.menu_Help.addAction(self.action_Project_page)
        self.menu_Help.addAction(self.action_Donate)
        self.menu_Help.addAction(self.action_Blog)
        self.menu_Help.addSeparator()
        self.menu_Help.addAction(self.actionAbout_Qt)
        self.menu_Help.addAction(self.action_About)
        self.menuBar.addAction(self.menu_File.menuAction())
        self.menuBar.addAction(self.menu_Help.menuAction())

        self.retranslateUi(MainWindow)
        self.action_Quit.triggered.connect(MainWindow.close)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "PdfHandoutCrop"))
        self.btnAutoDetect.setText(_translate("MainWindow", "&Auto detect"))
        self.label.setText(_translate("MainWindow", "Pages per sheet:"))
        self.btnReload.setText(_translate("MainWindow", "&Reload"))
        self.btnPrevious.setText(_translate("MainWindow", "&Previous", "previous page"))
        self.btnNext.setText(_translate("MainWindow", "&Next", "next page"))
        self.btnZoomIn.setText(_translate("MainWindow", "Zoom &in"))
        self.btnZoomOut.setText(_translate("MainWindow", "Zoom &out"))
        self.labelWidthHeight.setText(_translate("MainWindow", "Width and Height:"))
        self.btnWidthHeight.setText(_translate("MainWindow", "Click to set"))
        self.label_2.setText(_translate("MainWindow", "Position:"))
        self.label_3.setText(_translate("MainWindow", "x:", "x coordinate"))
        self.label_4.setText(_translate("MainWindow", "y:", "y coordinate"))
        self.btnPosition.setText(_translate("MainWindow", "Click to set"))
        self.menu_File.setTitle(_translate("MainWindow", "&File"))
        self.menu_Help.setTitle(_translate("MainWindow", "&Help"))
        self.action_Open.setText(_translate("MainWindow", "&Open"))
        self.action_Save.setText(_translate("MainWindow", "&Save"))
        self.actionAbout_Qt.setText(_translate("MainWindow", "About &Qt"))
        self.action_About.setText(_translate("MainWindow", "&About"))
        self.action_Quit.setText(_translate("MainWindow", "&Quit"))
        self.action_Website.setText(_translate("MainWindow", "&Website"))
        self.action_Donate.setText(_translate("MainWindow", "&Donate"))
        self.action_Project_page.setText(_translate("MainWindow", "&Project page"))
        self.action_Blog.setText(_translate("MainWindow", "&Blog"))




import copy
import os
from PyQt5.QtWidgets import QMainWindow, QFileDialog, QMessageBox, QDialog
from PyQt5.QtCore import pyqtSlot, Qt, QPoint, QEvent, QUrl
from PyQt5.QtGui import QPixmap, QPainter, QPainterPath, QIcon, QDesktopServices, QImage
import fitz
from PyPDF2 import PdfFileReader, PdfFileWriter
from pdfhandoutcrop.ui_mainwindow import Ui_MainWindow
from pdfhandoutcrop.setlayoutdialog import SetLayoutDialog

class MainWindow(QMainWindow, Ui_MainWindow):

    def __init__(self):
        super().__init__()
        self.setupUi(self)
        self.setWindowIcon(QIcon(os.path.dirname(os.path.abspath(__file__))+'/pdfhandoutcrop.png'))
        self.page_position=[[0, 0] for x in range(self.spbPagesPerSheet.value())]
        for i in range(self.spbPagesPerSheet.value()):
            self.comboPosition.addItem(self.tr("Page {0}").format(i+1))
        self.scaling = 2.0
        self.current_page=0
        #self.fileInput and self.fileOutput need to be defined first
        #because QFileDialog needs them
        self.fileInput=""
        self.fileOutput=""
        self.set=0
        self.needPaint=False
        self.labelSelectPoint.installEventFilter(self)

    @pyqtSlot()
    def on_action_Open_triggered(self):
        filename=QFileDialog.getOpenFileName(self, "", self.fileInput,
            self.tr("PDF documents (*.pdf)"))[0]
        if filename!="":
            self.fileInput=filename
            try:
                self.document=fitz.open(self.fileInput)
            except:
                QMessageBox.warning(self, self.tr("Error"), self.tr("Cannot open input file"))
                self.labelSelectPoint.setText("")
                return
            self.btnAutoDetect.setEnabled(True)
            self.btnReload.setEnabled(True)
            self.setWindowTitle(self.tr("{0} - PdfHandoutCrop").format(os.path.basename(self.fileInput)))
            self.current_page=0
            self.loadPdf()

    @pyqtSlot()
    def on_action_Save_triggered(self):
        filename=QFileDialog.getSaveFileName(self, "", self.fileOutput,
            self.tr("PDF documents (*.pdf)"))[0]
        if filename!="":
            self.fileOutput=filename
            self.save()

    @pyqtSlot()
    def on_action_Website_triggered(self):
        QDesktopServices.openUrl(QUrl("https://cges30901.github.io/pdfhandoutcrop/"))

    @pyqtSlot()
    def on_action_Project_page_triggered(self):
        QDesktopServices.openUrl(QUrl("https://github.com/cges30901/pdfhandoutcrop"))

    @pyqtSlot()
    def on_action_Donate_triggered(self):
        QDesktopServices.openUrl(QUrl("https://cges30901.github.io/pdfhandoutcrop/donate.html"))

    @pyqtSlot()
    def on_action_Blog_triggered(self):
        QDesktopServices.openUrl(QUrl("https://hsiu-ming.blogspot.com/"))

    @pyqtSlot()
    def on_actionAbout_Qt_triggered(self):
        QMessageBox.aboutQt(self)

    @pyqtSlot()
    def on_action_About_triggered(self):
        version="0.6.0"
        QMessageBox.about(self, self.tr("About"), self.tr(
'''<h3>PdfHandoutCrop {0}</h3><br>
Author: Hsiu-Ming Chang<br>
e-mail: cges30901@gmail.com<br>
License: GPL v3''').format(version))

    @pyqtSlot(int)
    def on_spbPagesPerSheet_valueChanged(self, num):
        while num>len(self.page_position):
            self.comboPosition.addItem(self.tr("Page {0}").format(len(self.page_position)+1))
            self.page_position.append([0, 0])
        while num<len(self.page_position):
            self.comboPosition.removeItem(len(self.page_position)-1)
            self.page_position.pop()
        self.needPaint=True
        self.update()

    def loadPdf(self):
        self.labelSelectPoint.setText(self.tr("Loading..."))
        self.labelSelectPoint.repaint()
        if self.current_page==0:
            self.btnPrevious.setEnabled(False)
        else:
            self.btnPrevious.setEnabled(True)
        if self.current_page==self.document.pageCount-1:
            self.btnNext.setEnabled(False)
        else:
            self.btnNext.setEnabled(True)
        self.page = self.document.loadPage(self.current_page)
        pix = self.page.getPixmap(fitz.Matrix(self.scaling, 0, 0, self.scaling, 0, 0), alpha = False)
        self.samples = pix.samples
        self.image=QImage(self.samples, pix.width, pix.height, pix.stride, QImage.Format_RGB888);
        self.pixmap=QPixmap.fromImage(self.image)
        self.needPaint=True
        self.labelSelectPoint.setPixmap(self.pixmap)
        self.labelPageNum.setText(str(self.current_page+1)+" / "+str(self.document.pageCount))

    def save(self):
        pdfInput=PdfFileReader(self.fileInput)
        pdfOutput=PdfFileWriter()
        numPages=pdfInput.getNumPages()
        pagesPerSheet=self.spbPagesPerSheet.value()
        width=self.spbWidth.value()/self.scaling
        height=self.spbHeight.value()/self.scaling
        sheetHeight=self.image.height()/self.scaling
        sheetWidth=self.image.width()/self.scaling
        #Getting mediaBox of page 0 directly makes output pages of first sheet
        #have same mediaBox, so I use copy.copy() to workaround this problem.
        page0=copy.copy(pdfInput.getPage(0))

        rotation=page0.get('/Rotate')

        #make rotation 0, 90, 180 or 270
        if rotation is None:
            rotation=0
        elif rotation<0:
            rotation=rotation%360+360

        #operation above might change rotation to 360,
        #so "if" is used instead of "elif"
        if rotation>=360:
            rotation=rotation%360

        #if lowerLeft of original mediaBox is not [0,0],
        #new mediaBox should be adjusted according to that.
        #FIXME: only fixed when page is not rotated currently.
        mediaBox_old=page0.mediaBox
        lowerLeftX_old=mediaBox_old.lowerLeft[0].as_numeric()
        lowerLeftY_old=mediaBox_old.lowerLeft[1].as_numeric()

        for i in range(numPages):
            page=pdfInput.getPage(i)
            for j in range(pagesPerSheet):
                page_crop=copy.copy(page)
                if rotation==90:
                    page_crop.mediaBox.lowerLeft=(sheetHeight-self.page_position[j][1]/self.scaling-height,
                        self.page_position[j][0]/self.scaling+width)
                    page_crop.mediaBox.upperRight=(sheetHeight-self.page_position[j][1]/self.scaling,
                        self.page_position[j][0]/self.scaling)
                elif rotation==180:
                    page_crop.mediaBox.lowerLeft=(sheetWidth-self.page_position[j][0]/self.scaling-width,
                        sheetHeight-self.page_position[j][1]/self.scaling)
                    page_crop.mediaBox.upperRight=(sheetWidth-self.page_position[j][0]/self.scaling,
                        sheetHeight-self.page_position[j][1]/self.scaling-height)
                elif rotation==270:
                    page_crop.mediaBox.lowerLeft=(self.page_position[j][1]/self.scaling,
                        sheetWidth-self.page_position[j][0]/self.scaling)
                    page_crop.mediaBox.upperRight=(self.page_position[j][1]/self.scaling+height,
                        sheetWidth-self.page_position[j][0]/self.scaling-width)
                else: #not rotated
                    page_crop.mediaBox.lowerLeft=(self.page_position[j][0]/self.scaling+lowerLeftX_old,
                        self.page_position[j][1]/self.scaling+height+lowerLeftY_old)
                    page_crop.mediaBox.upperRight=(self.page_position[j][0]/self.scaling+width+lowerLeftX_old,
                        self.page_position[j][1]/self.scaling+lowerLeftY_old)
                pdfOutput.addPage(page_crop)
        outputStream = open(self.fileOutput, "wb")
        pdfOutput.write(outputStream)
        outputStream.close()
        QMessageBox.information(self, self.tr("Finished"), self.tr("Cropped PDF saved"))

    @pyqtSlot(bool)
    def on_btnAutoDetect_clicked(self):
        #find the upperleft point of the first page
        pointUpperLeft=self.findFirstPoint()
        if pointUpperLeft.x()==-1:  #Page can not be found
            QMessageBox.warning(self, self.tr("Page can not be found"),
                self.tr("Page can not be found. Auto detect only works if pages have border."))
            return

        #find width
        width=100
        while pointUpperLeft.x()+width<self.image.width():
            if self.image.pixel(pointUpperLeft.x()+width, pointUpperLeft.y())==4294967295:
                break
            width+=1

        #find height
        height=100
        while pointUpperLeft.y()+height<self.image.height():
            if self.image.pixel(pointUpperLeft.x(), pointUpperLeft.y()+height)==4294967295:
                break
            height+=1

        #find columns
        columns=[pointUpperLeft.x()]
        iter=pointUpperLeft.x()+width
        while iter<self.image.width()-100:
            if self.image.pixel(iter, pointUpperLeft.y())!=4294967295:
                columns.append(iter)
                iter+=width
            iter+=1

        #find rows
        rows=[pointUpperLeft.y()]
        iter=pointUpperLeft.y()+height
        while iter<self.image.height()-100:
            if self.image.pixel(pointUpperLeft.x(), iter)!=4294967295:
                rows.append(iter)
                iter+=height
            iter+=1

        dlgSetLayout=SetLayoutDialog(self)
        if len(columns)!=1 or len(rows)!=1:
            dlgSetLayout.label.hide()
            dlgSetLayout.spbColumns.setValue(len(columns))
            dlgSetLayout.spbRows.setValue(len(rows))
            dlgSetLayout.spbColumns.setEnabled(False)
            dlgSetLayout.spbRows.setEnabled(False)
        if dlgSetLayout.exec_()==QDialog.Accepted:
            #ask columns and rows when only one page is detected (shared border)
            if len(columns)==1 and len(rows)==1:
                numColumns=dlgSetLayout.spbColumns.value()
                numRows=dlgSetLayout.spbRows.value()
                height/=numRows
                width/=numColumns
                for i in range(1, numColumns):
                    columns.append(columns[i-1]+width)
                for i in range(1, numRows):
                    rows.append(rows[i-1]+height)

        self.spbPagesPerSheet.setValue(len(rows)*len(columns))
        self.spbWidth.setValue(width)
        self.spbHeight.setValue(height)
        #point (0,0) is in lowerLeft, so coordinate need to be changed
        sheetHeight=self.image.height()
        pageHeight=self.spbHeight.value()

        if dlgSetLayout.comboOrder.currentIndex()==0:
            for i in range(len(rows)*len(columns)):
                self.page_position[i][0]=columns[i%len(columns)]
                self.page_position[i][1]=sheetHeight-pageHeight-rows[i//len(columns)]
        elif dlgSetLayout.comboOrder.currentIndex()==1:
            for i in range(len(rows)*len(columns)):
                self.page_position[i][0]=columns[len(columns)-1-i%len(columns)]
                self.page_position[i][1]=sheetHeight-pageHeight-rows[i//len(columns)]
        elif dlgSetLayout.comboOrder.currentIndex()==2:
            for i in range(len(rows)*len(columns)):
                self.page_position[i][0]=columns[i//len(rows)]
                self.page_position[i][1]=sheetHeight-pageHeight-rows[i%len(rows)]
        elif dlgSetLayout.comboOrder.currentIndex()==3:
            for i in range(len(rows)*len(columns)):
                self.page_position[i][0]=columns[len(columns)-1-i//len(rows)]
                self.page_position[i][1]=sheetHeight-pageHeight-rows[i%len(rows)]

        #update value of spbPositionX and spbPositionY
        self.spbPositionX.setValue(self.page_position[self.comboPosition.currentIndex()][0])
        self.spbPositionY.setValue(self.page_position[self.comboPosition.currentIndex()][1])
        self.needPaint=True
        self.update()
        self.statusBar.showMessage(self.tr("Found {0} pages").format(len(rows)*len(columns)), 2000)

    def findFirstPoint(self):
        for yOffset in range(self.image.height()-100):
            for xOffset in range(self.image.width()-100):
                pixel=self.image.pixel(xOffset, yOffset)
                #4294967295 is white
                if pixel!=4294967295:
                    for length in range(1, 101):
                        if (self.image.pixel(xOffset+length, yOffset)==4294967295
                            or self.image.pixel(xOffset, yOffset+length)==4294967295):
                            #not a page
                            break
                        if length==100:
                            #page found
                            return QPoint(xOffset, yOffset)
        #cannot find a page
        return QPoint(-1, -1)

    @pyqtSlot(int)
    def on_comboPosition_currentIndexChanged(self, num):
        self.spbPositionX.setValue(self.page_position[num][0])
        self.spbPositionY.setValue(self.page_position[num][1])

    @pyqtSlot(int)
    def on_spbPositionX_valueChanged(self, num):
        self.page_position[self.comboPosition.currentIndex()][0]=num

    @pyqtSlot(int)
    def on_spbPositionY_valueChanged(self, num):
        self.page_position[self.comboPosition.currentIndex()][1]=num

    @pyqtSlot(bool)
    def on_btnPosition_clicked(self):
        self.set=1
        self.statusBar.showMessage(self.tr(
            "Please click the upper left point of page {0}").format(self.comboPosition.currentIndex()+1))

    @pyqtSlot(bool)
    def on_btnWidthHeight_clicked(self):
        self.set=2
        self.statusBar.showMessage(self.tr("Please click the upper left point of any page"))

    def labelSelectPoint_mousePressed(self, x, y):
        if self.set==1:
            #self.spbHeight.value() should be set first for y coordinate
            if self.spbHeight.value()==0:
                QMessageBox.warning(self, self.tr("Warning"), self.tr("Height should be set first"))

            #set x coordinate
            self.spbPositionX.setValue(x)

            #point (0,0) is in lowerLeft, so y coordinate need to be changed
            sheetHeight=self.image.height()
            pageHeight=self.spbHeight.value()
            self.spbPositionY.setValue(sheetHeight-pageHeight-y)
            self.needPaint=True
            self.set=0
            self.statusBar.clearMessage()
        elif self.set==2:
            #set Width and Height - step one
            #store coordinate of upper left point
            self.upperleftX=x
            self.upperleftY=y
            #set lower right point if pressed again
            self.set=3
            self.statusBar.showMessage(self.tr("Please click the lower right point of the page you clicked"))
        elif self.set==3:
            #set Width and Height - step two
            #get coordinate of lower right point (x,y)
            #and calculate width and height
            self.spbWidth.setValue(x-self.upperleftX)
            self.spbHeight.setValue(y-self.upperleftY)
            self.needPaint=True
            self.set=0
            self.statusBar.clearMessage()
        self.update()

    def paintEvent(self, e):
        if self.needPaint:
            pixmap_draw=QPixmap(self.pixmap)
            painter=QPainter()
            painter.begin(pixmap_draw)
            painter.setPen(Qt.red)
            path=[QPainterPath() for i in range(self.spbPagesPerSheet.value())]
            for i in range(self.spbPagesPerSheet.value()):
                sheetHeight=self.image.height()
                pageHeight=self.spbHeight.value()
                path[i].moveTo(self.page_position[i][0],
                    sheetHeight-pageHeight-self.page_position[i][1])
                path[i].lineTo(self.page_position[i][0]+self.spbWidth.value(),
                    sheetHeight-pageHeight-self.page_position[i][1])
                path[i].lineTo(self.page_position[i][0]+self.spbWidth.value(),
                    sheetHeight-pageHeight-self.page_position[i][1]+self.spbHeight.value())
                path[i].lineTo(self.page_position[i][0],
                    sheetHeight-pageHeight-self.page_position[i][1]+self.spbHeight.value())
                path[i].lineTo(self.page_position[i][0],
                    sheetHeight-pageHeight-self.page_position[i][1])
                painter.drawPath(path[i])
            painter.end()
            self.labelSelectPoint.setPixmap(pixmap_draw)
            self.needPaint=False

    @pyqtSlot(bool)
    def on_btnReload_clicked(self):
        self.needPaint=True
        self.update()

    @pyqtSlot(bool)
    def on_btnPrevious_clicked(self):
        self.current_page-=1
        self.loadPdf()
        self.needPaint=True
        self.update()

    @pyqtSlot(bool)
    def on_btnNext_clicked(self):
        self.current_page+=1
        self.loadPdf()
        self.needPaint=True
        self.update()

    def on_btnZoomIn_clicked(self):
        self.scaling *= 1.2
        self.needPaint=True
        self.loadPdf()

    def on_btnZoomOut_clicked(self):
        self.scaling /= 1.2
        self.needPaint=True
        self.loadPdf()

    def eventFilter(self, watched, event):
        if watched==self.labelSelectPoint and event.type()==QEvent.MouseButtonPress:
            self.labelSelectPoint_mousePressed(event.x(), event.y())
        return False



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pdfhandoutcrop',
    version='0.6.0',
    description='A tool to crop pdf handout with multiple pages per sheet',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://cges30901.github.io/pdfhandoutcrop/',
    download_url='https://cges30901.github.io/pdfhandoutcrop/download.html',
    author='Hsiu-Ming Chang',
    author_email='cges30901@gmail.com',
    license='GPLv3+',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
        'Topic :: Education',
        'Topic :: Office/Business',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
        'Natural Language :: English',
        'Natural Language :: Chinese (Traditional)',
    ],
    keywords='pdf crop',
    project_urls={
        'Bug Reports': 'https://github.com/cges30901/pdfhandoutcrop/issues',
        'Source': 'https://github.com/cges30901/pdfhandoutcrop/',
        'Donate': 'https://cges30901.github.io/pdfhandoutcrop/donate.html',
    },
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=['PyQt5', 'PyMuPDF', 'PyPDF2'],
    python_requires='>=3',
    scripts=['scripts/pdfhandoutcrop'],
    package_data={
        'pdfhandoutcrop': ['pdfhandoutcrop.png', 'language/*.ts', 'language/*.qm'],
    },
)



from pdfhandoutcrop.ui_setlayoutdialog import Ui_SetLayoutDialog
from PyQt5.QtWidgets import QDialog

class SetLayoutDialog(QDialog, Ui_SetLayoutDialog):

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.comboOrder.addItem(self.tr("left to right, then down"))
        self.comboOrder.addItem(self.tr("right to left, then down"))
        self.comboOrder.addItem(self.tr("top to bottom, then right"))
        self.comboOrder.addItem(self.tr("top to bottom, then left"))

    def on_pushButton_clicked(self):
        self.accept()
